@@ -14,12 +14,12 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-from functools import cache
 from types import SimpleNamespace
 
 import torch
 
-from .jit import gen_act_and_mul_module, has_prebuilt_ops, load_cuda_ops  # noqa: F401
+from .jit import JitSpec
+from .jit import gen_act_and_mul_module as gen_act_and_mul_module_impl
 from .utils import register_custom_op, register_fake_op
 
 silu_def_cu_str = r"""
@@ -53,17 +53,14 @@ act_func_def_str = {
 _jit_modules = {}
 
 
+def gen_act_and_mul_module(act_func_name: str) -> JitSpec:
+    return gen_act_and_mul_module_impl(act_func_name, act_func_def_str[act_func_name])
+
+
 def get_act_and_mul_module(act_func_name: str):
     global _jit_modules
     if act_func_name not in _jit_modules:
-        if has_prebuilt_ops:
-            _kernels = torch.ops.flashinfer_kernels
-
-            module = _kernels
-        else:
-            module = gen_act_and_mul_module(
-                act_func_name, act_func_def_str[act_func_name]
-            )
+        module = gen_act_and_mul_module(act_func_name).build_and_load()
 
         # torch library for act_and_mul
         fname = f"{act_func_name}_and_mul"
