@@ -20,28 +20,29 @@ from typing import Any, List, Optional, Tuple
 import torch
 
 from .decode import BatchDecodeWithPagedKVCacheWrapper
-from .jit import FLASHINFER_CSRC_DIR, has_prebuilt_ops, load_cuda_ops
+from .jit import JitSpec
+from .jit import env as jit_env
+from .jit import gen_jit_spec
 from .prefill import BatchPrefillWithPagedKVCacheWrapper, single_prefill_with_kv_cache
 from .utils import register_custom_op, register_fake_op
 
 _cascade_module = None
 
 
+def gen_cascade_module() -> JitSpec:
+    return gen_jit_spec(
+        "cascade",
+        [
+            jit_env.FLASHINFER_CSRC_DIR / "cascade.cu",
+            jit_env.FLASHINFER_CSRC_DIR / "flashinfer_cascade_ops.cu",
+        ],
+    )
+
+
 def get_cascade_module():
     global _cascade_module
     if _cascade_module is None:
-        if has_prebuilt_ops:
-            _kernels = torch.ops.flashinfer_kernels
-
-            _cascade_module = _kernels
-        else:
-            _cascade_module = load_cuda_ops(
-                "cascade",
-                [
-                    FLASHINFER_CSRC_DIR / "cascade.cu",
-                    FLASHINFER_CSRC_DIR / "flashinfer_cascade_ops.cu",
-                ],
-            )
+        _cascade_module = gen_cascade_module().build_and_load()
     return _cascade_module
 
 
@@ -568,7 +569,7 @@ class BatchDecodeWithSharedPrefixPagedKVCacheWrapper:
     of requests. The shared-prefix KV-Cache was stored in a standalone tensors, and the
     unique KV-Cache of each request was stored in a paged KV-Cache data structure.
 
-    Check :ref:`our tutorial<page-layout>` for page table layout.
+    Check :ref:`our tutorial<kv-layout>` for page table layout.
 
     Warning
     -------
@@ -807,7 +808,7 @@ class BatchPrefillWithSharedPrefixPagedKVCacheWrapper:
     r"""Wrapper class for prefill/append attention with shared-prefix paged kv-cache for
     batch of requests.
 
-    Check :ref:`our tutorial<page-layout>` for paged kv-cache layout.
+    Check :ref:`our tutorial<kv-layout>` for paged kv-cache layout.
 
     Warning
     -------
