@@ -18,6 +18,8 @@ import ctypes
 import os
 
 # Re-export
+from . import cubin_loader
+from . import env as env
 from .activation import gen_act_and_mul_module as gen_act_and_mul_module
 from .activation import get_act_and_mul_cu_str as get_act_and_mul_cu_str
 from .attention import gen_batch_decode_mla_module as gen_batch_decode_mla_module
@@ -43,6 +45,7 @@ from .attention import (
 from .attention import (
     gen_customize_single_prefill_module as gen_customize_single_prefill_module,
 )
+from .attention import gen_fmha_cutlass_sm100a_module as gen_fmha_cutlass_sm100a_module
 from .attention import gen_pod_module as gen_pod_module
 from .attention import gen_sampling_tvm_binding as gen_sampling_tvm_binding
 from .attention import gen_single_decode_module as gen_single_decode_module
@@ -54,28 +57,17 @@ from .attention import get_batch_prefill_uri as get_batch_prefill_uri
 from .attention import get_pod_uri as get_pod_uri
 from .attention import get_single_decode_uri as get_single_decode_uri
 from .attention import get_single_prefill_uri as get_single_prefill_uri
-from .core import clear_cache_dir, load_cuda_ops  # noqa: F401
-from .env import *
-from .utils import parallel_load_modules as parallel_load_modules
+from .attention import trtllm_fmha_gen_module as trtllm_fmha_gen_module
+from .core import JitSpec as JitSpec
+from .core import build_jit_specs as build_jit_specs
+from .core import clear_cache_dir as clear_cache_dir
+from .core import gen_jit_spec as gen_jit_spec
+from .core import sm90a_nvcc_flags as sm90a_nvcc_flags
+from .core import sm100a_nvcc_flags as sm100a_nvcc_flags
+from .cubin_loader import setup_cubin_loader
 
 cuda_lib_path = os.environ.get(
     "CUDA_LIB_PATH", "/usr/local/cuda/targets/x86_64-linux/lib/"
 )
 if os.path.exists(f"{cuda_lib_path}/libcudart.so.12"):
     ctypes.CDLL(f"{cuda_lib_path}/libcudart.so.12", mode=ctypes.RTLD_GLOBAL)
-
-
-try:
-    from .. import flashinfer_kernels, flashinfer_kernels_sm90  # noqa: F401
-    from .aot_config import prebuilt_ops_uri as prebuilt_ops_uri
-
-    has_prebuilt_ops = True
-except ImportError as e:
-    if "undefined symbol" in str(e):
-        raise ImportError("Loading prebuilt ops failed.") from e
-
-    from .core import logger
-
-    logger.info("Prebuilt kernels not found, using JIT backend")
-    prebuilt_ops_uri = {}
-    has_prebuilt_ops = False
