@@ -20,8 +20,8 @@ from typing import List
 import jinja2
 import torch
 
-from ..core import load_cuda_ops, logger, sm90a_nvcc_flags
-from ..env import FLASHINFER_CSRC_DIR, FLASHINFER_GEN_SRC_DIR
+from .. import env as jit_env
+from ..core import JitSpec, gen_jit_spec, logger, sm90a_nvcc_flags, sm100a_nvcc_flags
 from ..utils import (
     dtype_map,
     filename_safe_dtype_map,
@@ -108,7 +108,7 @@ def gen_batch_mla_module(
     head_dim_ckv: int,
     head_dim_kpe: int,
     use_profiler: bool,
-):
+) -> JitSpec:
     if backend == "auto":
         raise ValueError("backend should not be auto when jit_args is provided")
     uri = get_batch_mla_uri(
@@ -121,11 +121,11 @@ def gen_batch_mla_module(
         head_dim_kpe,
         use_profiler,
     )
-    gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+    gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
     os.makedirs(gen_directory, exist_ok=True)
 
     if backend == "fa2":
-        with open(FLASHINFER_CSRC_DIR / "batch_mla_config.jinja") as f:
+        with open(jit_env.FLASHINFER_CSRC_DIR / "batch_mla_config.jinja") as f:
             config_templ = jinja2.Template(f.read())
         generated_config_path = gen_directory / "batch_mla_config.inc"
         write_if_different(
@@ -146,14 +146,14 @@ def gen_batch_mla_module(
             "batch_mla_run.cu",
             "batch_mla_pybind.cu",
         ]:
-            src_path = FLASHINFER_CSRC_DIR / filename
+            src_path = jit_env.FLASHINFER_CSRC_DIR / filename
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
             with open(src_path, "r") as f:
                 source = f.read()
             write_if_different(dest_path, source)
     elif backend == "fa3":
-        with open(FLASHINFER_CSRC_DIR / "batch_mla_config.jinja") as f:
+        with open(jit_env.FLASHINFER_CSRC_DIR / "batch_mla_config.jinja") as f:
             config_templ = jinja2.Template(f.read())
         generated_config_path = gen_directory / "batch_mla_sm90_config.inc"
         write_if_different(
@@ -173,7 +173,7 @@ def gen_batch_mla_module(
             "batch_mla_sm90_run.cu",
             "batch_mla_sm90_pybind.cu",
         ]:
-            src_path = FLASHINFER_CSRC_DIR / filename
+            src_path = jit_env.FLASHINFER_CSRC_DIR / filename
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
             with open(src_path, "r") as f:
@@ -182,13 +182,16 @@ def gen_batch_mla_module(
     else:
         raise ValueError(f"Unsupported backend: {backend}")
 
-    return load_cuda_ops(
+    extra_cuda_cflags = []
+    if backend == "fa3":
+        extra_cuda_cflags += sm90a_nvcc_flags
+    if use_profiler:
+        extra_cuda_cflags += ["-DFLASHINFER_ENABLE_PROFILER"]
+
+    return gen_jit_spec(
         uri,
         source_paths,
-        extra_cuda_cflags=(
-            ["-gencode=arch=compute_90a,code=sm_90a"] if backend == "fa3" else []
-        )
-        + (["-DFLASHINFER_ENABLE_PROFILER"] if use_profiler else []),
+        extra_cuda_cflags=extra_cuda_cflags,
     )
 
 
@@ -224,7 +227,7 @@ def gen_batch_decode_mla_module(
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
     use_tensor_cores: bool,
-):
+) -> JitSpec:
     cuda_arch_major = torch.cuda.get_device_properties(0).major
 
     if cuda_arch_major >= 9:  # smem size of SM90 can accommodate all 128 qo-heads data
@@ -240,10 +243,10 @@ def gen_batch_decode_mla_module(
         and dtype_kv == torch.float16
         and dtype_o == torch.float16
     ):
-        logger.info(f"Use tensor-core SM80 version of MLA decode kernel.")
+        logger.info("Use tensor-core SM80 version of MLA decode kernel.")
         arc = "sm80"
     else:
-        logger.info(f"Fall back to cuda-core version of MLA decode kernel.")
+        logger.info("Fall back to cuda-core version of MLA decode kernel.")
         arc = "cuda_core"
 
     uri = get_batch_decode_mla_uri(
@@ -256,10 +259,10 @@ def gen_batch_decode_mla_module(
         use_logits_soft_cap,
         arc,
     )
-    gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+    gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
     os.makedirs(gen_directory, exist_ok=True)
 
-    with open(FLASHINFER_CSRC_DIR / "batch_decode_mla_config.jinja") as f:
+    with open(jit_env.FLASHINFER_CSRC_DIR / "batch_decode_mla_config.jinja") as f:
         config_templ = jinja2.Template(f.read())
     generated_config_path = gen_directory / "mla_config.inc"
     write_if_different(
@@ -292,14 +295,14 @@ def gen_batch_decode_mla_module(
 
     source_paths = []
     for filename in filenames:
-        src_path = FLASHINFER_CSRC_DIR / filename
+        src_path = jit_env.FLASHINFER_CSRC_DIR / filename
         dest_path = gen_directory / filename
         source_paths.append(dest_path)
         with open(src_path, "r") as f:
             source = f.read()
         write_if_different(dest_path, source)
 
-    return load_cuda_ops(uri, source_paths)
+    return gen_jit_spec(uri, source_paths)
 
 
 def get_single_prefill_uri(
@@ -393,7 +396,7 @@ def gen_single_decode_module(
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
-):
+) -> JitSpec:
     uri = get_single_decode_uri(
         dtype_q,
         dtype_kv,
@@ -421,7 +424,7 @@ def gen_single_decode_module(
         ],  # additional_scalar_names
         ["double", "double", "double", "double"],  # additional_scalar_dtypes
         f"DefaultAttention<false, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>",  # variant_name
-        f"#include<flashinfer/attention/variants.cuh>",  # variant_decl
+        "#include<flashinfer/attention/variants.cuh>",  # variant_decl
         pos_encoding_mode=pos_encoding_mode,
         use_sliding_window=use_sliding_window,
         use_logits_soft_cap=use_logits_soft_cap,
@@ -439,7 +442,7 @@ def gen_single_prefill_module(
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
     use_fp16_qk_reduction: bool,
-):
+) -> JitSpec:
     uri = get_single_prefill_uri(
         backend,
         dtype_q,
@@ -452,7 +455,14 @@ def gen_single_prefill_module(
         use_logits_soft_cap,
         use_fp16_qk_reduction,
     )
+
+    # use `fp8_enabled` flag to use separate kernel template
+    # this is used for fp8 tensor core computation
+    # KV-only quant is not influenced by this flag
+    fp8_enabled = dtype_q in [torch.float8_e4m3fn, torch.float8_e5m2]
+
     if backend == "fa2":
+        assert not fp8_enabled, "fp8 tensor core is not supported in fa2 backend"
         additional_tensor_names = ["maybe_custom_mask", "maybe_alibi_slopes"]
         additional_tensor_dtypes = ["uint8_t", "float"]
         additional_scalar_names = [
@@ -463,14 +473,22 @@ def gen_single_prefill_module(
         ]
         additional_scalar_dtypes = ["double", "double", "double", "double"]
         variant_name = f"DefaultAttention<use_custom_mask, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>"
-        variant_decl = f"#include<flashinfer/attention/variants.cuh>"
+        variant_decl = "#include<flashinfer/attention/variants.cuh>"
     else:
-        additional_tensor_names = []
-        additional_tensor_dtypes = []
-        additional_scalar_names = ["logits_soft_cap", "sm_scale"]
-        additional_scalar_dtypes = ["double", "double"]
-        variant_name = f"DefaultAttention<{str(use_logits_soft_cap).lower()}>"
-        variant_decl = f"#include<flashinfer/attention/hopper/variants.cuh>"
+        if not fp8_enabled:
+            additional_tensor_names = []
+            additional_tensor_dtypes = []
+            additional_scalar_names = ["logits_soft_cap", "sm_scale"]
+            additional_scalar_dtypes = ["double", "double"]
+            variant_name = f"DefaultAttention<{str(use_logits_soft_cap).lower()}>"
+            variant_decl = "#include<flashinfer/attention/hopper/variants.cuh>"
+        else:
+            additional_tensor_names = ["scale_q", "scale_k", "scale_v"]
+            additional_tensor_dtypes = ["float", "float", "float"]
+            additional_scalar_names = ["sm_scale"]
+            additional_scalar_dtypes = ["double"]
+            variant_name = "DefaultFP8Attention"
+            variant_decl = "#include<flashinfer/attention/hopper/variants.cuh>"
 
     return gen_customize_single_prefill_module(
         backend,
@@ -490,6 +508,7 @@ def gen_single_prefill_module(
         use_sliding_window=use_sliding_window,
         use_logits_soft_cap=use_logits_soft_cap,
         use_fp16_qk_reduction=use_fp16_qk_reduction,
+        fp8_enabled=fp8_enabled,
     )
 
 
@@ -506,7 +525,7 @@ def gen_pod_module(
     pos_encoding_mode_d: int,
     use_sliding_window_d: bool,
     use_logits_soft_cap_d: bool,
-):
+) -> JitSpec:
     uri = get_pod_uri(
         dtype_q,
         dtype_kv,
@@ -532,7 +551,7 @@ def gen_pod_module(
     additional_scalar_dtypes = ["float", "float", "float", "float"]
     variant_name_p = f"DefaultAttention<use_custom_mask_p, {str(use_sliding_window_p).lower()}, {str(use_logits_soft_cap_p).lower()}, {str(pos_encoding_mode_p == 2).lower()}>"
     variant_name_d = f"DefaultAttention<use_custom_mask_d, {str(use_sliding_window_d).lower()}, {str(use_logits_soft_cap_d).lower()}, {str(pos_encoding_mode_d == 2).lower()}>"
-    variant_decl = f"#include<flashinfer/attention/variants.cuh>"
+    variant_decl = "#include<flashinfer/attention/variants.cuh>"
 
     return gen_customize_pod_module(
         uri,
@@ -579,8 +598,8 @@ def gen_customize_pod_module(
     use_sliding_window_d: bool = False,
     use_logits_soft_cap_d: bool = False,
     use_fp16_qk_reduction: bool = False,
-):
-    gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+) -> JitSpec:
+    gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
 
     (
         additional_params_decl,
@@ -593,10 +612,10 @@ def gen_customize_pod_module(
         additional_scalar_dtypes,
     )
 
-    with open(FLASHINFER_CSRC_DIR / "pod_customize_config.jinja") as f:
+    with open(jit_env.FLASHINFER_CSRC_DIR / "pod_customize_config.jinja") as f:
         config_templ = jinja2.Template(f.read())
 
-    with open(FLASHINFER_CSRC_DIR / "pod_kernel_inst.jinja") as f:
+    with open(jit_env.FLASHINFER_CSRC_DIR / "pod_kernel_inst.jinja") as f:
         kernel_inst_templ = jinja2.Template(f.read())
 
     kwargs = {
@@ -629,8 +648,8 @@ def gen_customize_pod_module(
 
     source_paths = []
 
-    for mask_mode_p in [0, 1, 2]:
-        for mask_mode_d in [0, 1, 2]:
+    for mask_mode_p in [0, 1, 2, 3]:
+        for mask_mode_d in [0, 1, 2, 3]:
             kwargs["mask_mode_p"] = mask_mode_literal[mask_mode_p]
             kwargs["mask_mode_d"] = mask_mode_literal[mask_mode_d]
 
@@ -646,7 +665,7 @@ def gen_customize_pod_module(
         "pod.cu",
         "pod_jit_pybind.cu",
     ]:
-        src_path = FLASHINFER_CSRC_DIR / filename
+        src_path = jit_env.FLASHINFER_CSRC_DIR / filename
         dest_path = gen_directory / filename
         source_paths.append(dest_path)
         with open(src_path, "r") as f:
@@ -655,7 +674,7 @@ def gen_customize_pod_module(
 
     generated_config_path = gen_directory / "pod_config.inc"
     write_if_different(generated_config_path, generated_inc_str)
-    return load_cuda_ops(uri, source_paths)
+    return gen_jit_spec(uri, source_paths)
 
 
 def gen_batch_decode_module(
@@ -668,7 +687,7 @@ def gen_batch_decode_module(
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
-):
+) -> JitSpec:
     uri = get_batch_decode_uri(
         dtype_q,
         dtype_kv,
@@ -698,7 +717,7 @@ def gen_batch_decode_module(
         ],  # additional_scalar_names
         ["double", "double", "double", "double"],  # additional_scalar_dtypes
         f"DefaultAttention<false, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>",  # variant_name
-        f"#include<flashinfer/attention/variants.cuh>",  # variant_decl
+        "#include<flashinfer/attention/variants.cuh>",  # variant_decl
         pos_encoding_mode=pos_encoding_mode,
         use_sliding_window=use_sliding_window,
         use_logits_soft_cap=use_logits_soft_cap,
@@ -717,7 +736,7 @@ def gen_batch_prefill_module(
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
     use_fp16_qk_reduction: bool,
-):
+) -> JitSpec:
     uri = get_batch_prefill_uri(
         backend,
         dtype_q,
@@ -732,33 +751,62 @@ def gen_batch_prefill_module(
         use_fp16_qk_reduction,
     )
 
+    # use `fp8_enabled` flag to use separate kernel template
+    # this is used for fp8 tensor core computation
+    # KV-only quant is not influenced by this flag
+    fp8_enabled = dtype_q in [torch.float8_e4m3fn, torch.float8_e5m2]
+
     if backend == "fa2":
+        assert not fp8_enabled, "fp8 tensor core is not supported in fa2 backend"
         additional_tensor_names = [
             "maybe_custom_mask",
             "maybe_mask_indptr",
             "maybe_alibi_slopes",
+            "maybe_prefix_len_ptr",
+            "maybe_token_pos_in_items_ptr",
+            "maybe_max_item_len_ptr",
         ]
         additional_tensor_dtypes = [
             "uint8_t",
             "int32_t",
             "float",
+            "uint32_t",
+            "uint16_t",
+            "uint16_t",
         ]  # NOTE(Zihao): int32_t should follow dtype_idx
         additional_scalar_names = [
             "logits_soft_cap",
             "sm_scale",
             "rope_rcp_scale",
             "rope_rcp_theta",
+            "token_pos_in_items_len",
         ]
-        additional_scalar_dtypes = ["double", "double", "double", "double"]
+        additional_scalar_dtypes = ["double", "double", "double", "double", "int64_t"]
         variant_name = f"DefaultAttention<use_custom_mask, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>"
-        variant_decl = f"#include<flashinfer/attention/variants.cuh>"
+        variant_decl = "#include<flashinfer/attention/variants.cuh>"
     else:
-        additional_tensor_names = []
-        additional_tensor_dtypes = []
-        additional_scalar_names = ["logits_soft_cap", "sm_scale"]
-        additional_scalar_dtypes = ["double", "double"]
-        variant_name = f"DefaultAttention<{str(use_logits_soft_cap).lower()}>"
-        variant_decl = f"#include<flashinfer/attention/hopper/variants.cuh>"
+        if not fp8_enabled:
+            additional_tensor_names = [
+                "maybe_prefix_len_ptr",
+                "maybe_token_pos_in_items_ptr",
+                "maybe_max_item_len_ptr",
+            ]
+            additional_tensor_dtypes = ["uint32_t", "uint16_t", "uint16_t"]
+            additional_scalar_names = [
+                "logits_soft_cap",
+                "sm_scale",
+                "token_pos_in_items_len",
+            ]
+            additional_scalar_dtypes = ["double", "double", "int64_t"]
+            variant_name = f"DefaultAttention<{str(use_logits_soft_cap).lower()}>"
+            variant_decl = "#include<flashinfer/attention/hopper/variants.cuh>"
+        else:
+            additional_tensor_names = ["scale_q", "scale_k", "scale_v"]
+            additional_tensor_dtypes = ["float", "float", "float"]
+            additional_scalar_names = ["sm_scale"]
+            additional_scalar_dtypes = ["double"]
+            variant_name = "DefaultFP8Attention"
+            variant_decl = "#include<flashinfer/attention/hopper/variants.cuh>"
 
     return gen_customize_batch_prefill_module(
         backend,
@@ -779,6 +827,7 @@ def gen_batch_prefill_module(
         use_sliding_window=use_sliding_window,
         use_logits_soft_cap=use_logits_soft_cap,
         use_fp16_qk_reduction=use_fp16_qk_reduction,
+        fp8_enabled=fp8_enabled,
     )
 
 
@@ -798,8 +847,8 @@ def gen_customize_single_decode_module(
     pos_encoding_mode: int = 0,
     use_sliding_window: bool = False,
     use_logits_soft_cap: bool = False,
-):
-    gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+) -> JitSpec:
+    gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
 
     (
         additional_params_decl,
@@ -812,10 +861,12 @@ def gen_customize_single_decode_module(
         additional_scalar_dtypes,
     )
 
-    with open(FLASHINFER_CSRC_DIR / "single_decode_customize_config.jinja") as f:
+    with open(
+        jit_env.FLASHINFER_CSRC_DIR / "single_decode_customize_config.jinja"
+    ) as f:
         config_templ = jinja2.Template(f.read())
 
-    with open(FLASHINFER_CSRC_DIR / "single_decode_kernel_inst.jinja") as f:
+    with open(jit_env.FLASHINFER_CSRC_DIR / "single_decode_kernel_inst.jinja") as f:
         kernel_inst_templ = jinja2.Template(f.read())
 
     kwargs = {
@@ -853,7 +904,7 @@ def gen_customize_single_decode_module(
         "single_decode.cu",
         "single_decode_jit_pybind.cu",
     ]:
-        src_path = FLASHINFER_CSRC_DIR / filename
+        src_path = jit_env.FLASHINFER_CSRC_DIR / filename
         dest_path = gen_directory / filename
         source_paths.append(dest_path)
         with open(src_path, "r") as f:
@@ -863,7 +914,7 @@ def gen_customize_single_decode_module(
     generated_config_path = gen_directory / "single_decode_config.inc"
     write_if_different(generated_config_path, generated_inc_str)
 
-    return load_cuda_ops(uri, source_paths)
+    return gen_jit_spec(uri, source_paths)
 
 
 def gen_customize_single_prefill_module(
@@ -884,7 +935,8 @@ def gen_customize_single_prefill_module(
     use_sliding_window: bool = False,
     use_logits_soft_cap: bool = False,
     use_fp16_qk_reduction: bool = False,
-):
+    fp8_enabled: bool = False,
+) -> JitSpec:
     kwargs = {
         "variant_decl": variant_decl,
         "variant_name": variant_name,
@@ -901,7 +953,7 @@ def gen_customize_single_prefill_module(
     if backend == "auto":
         raise ValueError("backend should not be auto when jit_args is provided")
     elif backend == "fa2":
-        gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+        gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
         additional_params_decl, additional_func_params, additional_params_setter = (
             generate_additional_params(
                 additional_tensor_names,
@@ -911,10 +963,14 @@ def gen_customize_single_prefill_module(
             )
         )
 
-        with open(FLASHINFER_CSRC_DIR / "single_prefill_customize_config.jinja") as f:
+        with open(
+            jit_env.FLASHINFER_CSRC_DIR / "single_prefill_customize_config.jinja"
+        ) as f:
             config_templ = jinja2.Template(f.read())
 
-        with open(FLASHINFER_CSRC_DIR / "single_prefill_kernel_inst.jinja") as f:
+        with open(
+            jit_env.FLASHINFER_CSRC_DIR / "single_prefill_kernel_inst.jinja"
+        ) as f:
             kernel_inst_templ = jinja2.Template(f.read())
 
         kwargs |= {
@@ -929,7 +985,7 @@ def gen_customize_single_prefill_module(
         os.makedirs(gen_directory, exist_ok=True)
 
         source_paths = []
-        for mask_mode in [0, 1, 2]:
+        for mask_mode in [0, 1, 2, 3]:
             filename = f"single_prefill_kernel_mask_{mask_mode}.cu"
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
@@ -943,7 +999,7 @@ def gen_customize_single_prefill_module(
             "single_prefill.cu",
             "single_prefill_jit_pybind.cu",
         ]:
-            src_path = FLASHINFER_CSRC_DIR / filename
+            src_path = jit_env.FLASHINFER_CSRC_DIR / filename
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
             with open(src_path, "r") as f:
@@ -953,9 +1009,9 @@ def gen_customize_single_prefill_module(
         generated_config_path = gen_directory / "single_prefill_config.inc"
         write_if_different(generated_config_path, generated_inc_str)
 
-        return load_cuda_ops(uri, source_paths)
+        return gen_jit_spec(uri, source_paths)
     elif backend == "fa3":
-        gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+        gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
 
         (additional_params_decl, additional_func_params, additional_params_setter) = (
             generate_additional_params(
@@ -967,12 +1023,18 @@ def gen_customize_single_prefill_module(
             )
         )
 
-        with open(
-            FLASHINFER_CSRC_DIR / "single_prefill_sm90_customize_config.jinja"
-        ) as f:
+        _file_config = "single_prefill_sm90_customize_config.jinja"
+        if fp8_enabled:
+            _file_kernel_inst = "single_prefill_fp8_sm90_kernel_inst.jinja"
+            _file_csrc = "single_prefill_fp8_sm90.cu"
+        else:
+            _file_kernel_inst = "single_prefill_sm90_kernel_inst.jinja"
+            _file_csrc = "single_prefill_sm90.cu"
+
+        with open(jit_env.FLASHINFER_CSRC_DIR / _file_config) as f:
             config_templ = jinja2.Template(f.read())
 
-        with open(FLASHINFER_CSRC_DIR / "single_prefill_sm90_kernel_inst.jinja") as f:
+        with open(jit_env.FLASHINFER_CSRC_DIR / _file_kernel_inst) as f:
             kernel_inst_templ = jinja2.Template(f.read())
 
         kwargs |= {
@@ -987,7 +1049,7 @@ def gen_customize_single_prefill_module(
         os.makedirs(gen_directory, exist_ok=True)
 
         source_paths = []
-        for mask_mode in [0, 1, 2]:
+        for mask_mode in [0, 1, 2, 3]:
             filename = f"single_prefill_sm90_kernel_mask_{mask_mode}.cu"
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
@@ -998,10 +1060,10 @@ def gen_customize_single_prefill_module(
             write_if_different(dest_path, source)
 
         for filename in [
-            "single_prefill_sm90.cu",
+            _file_csrc,
             "single_prefill_sm90_jit_pybind.cu",
         ]:
-            src_path = FLASHINFER_CSRC_DIR / filename
+            src_path = jit_env.FLASHINFER_CSRC_DIR / filename
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
             with open(src_path, "r") as f:
@@ -1010,10 +1072,10 @@ def gen_customize_single_prefill_module(
 
         generated_config_path = gen_directory / "single_prefill_sm90_config.inc"
         write_if_different(generated_config_path, generated_inc_str)
-        return load_cuda_ops(
+        return gen_jit_spec(
             uri,
             source_paths,
-            extra_cuda_cflags=["-gencode=arch=compute_90a,code=sm_90a"],
+            extra_cuda_cflags=sm90a_nvcc_flags,
         )
     else:
         raise ValueError(f"Invalid backend: {backend}")
@@ -1036,8 +1098,8 @@ def gen_customize_batch_decode_module(
     pos_encoding_mode: int = 0,
     use_sliding_window: bool = False,
     use_logits_soft_cap: bool = False,
-):
-    gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+) -> JitSpec:
+    gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
     (additional_params_decl, additional_func_params, additional_params_setter) = (
         generate_additional_params(
             additional_tensor_names,
@@ -1064,10 +1126,10 @@ def gen_customize_batch_decode_module(
         "use_logits_soft_cap": str(use_logits_soft_cap).lower(),
     }
 
-    with open(FLASHINFER_CSRC_DIR / "batch_decode_customize_config.jinja") as f:
+    with open(jit_env.FLASHINFER_CSRC_DIR / "batch_decode_customize_config.jinja") as f:
         config_templ = jinja2.Template(f.read())
 
-    with open(FLASHINFER_CSRC_DIR / "batch_decode_kernel_inst.jinja") as f:
+    with open(jit_env.FLASHINFER_CSRC_DIR / "batch_decode_kernel_inst.jinja") as f:
         kernel_inst_templ = jinja2.Template(f.read())
 
     generated_inc_str = config_templ.render(
@@ -1087,7 +1149,7 @@ def gen_customize_batch_decode_module(
         "batch_decode.cu",
         "batch_decode_jit_pybind.cu",
     ]:
-        src_path = FLASHINFER_CSRC_DIR / filename
+        src_path = jit_env.FLASHINFER_CSRC_DIR / filename
         dest_path = gen_directory / filename
         source_paths.append(dest_path)
         with open(src_path, "r") as f:
@@ -1096,10 +1158,7 @@ def gen_customize_batch_decode_module(
 
     generated_config_path = gen_directory / "batch_decode_config.inc"
     write_if_different(generated_config_path, generated_inc_str)
-    return load_cuda_ops(
-        uri,
-        source_paths,
-    )
+    return gen_jit_spec(uri, source_paths)
 
 
 def gen_customize_batch_prefill_module(
@@ -1121,7 +1180,8 @@ def gen_customize_batch_prefill_module(
     use_sliding_window: bool = False,
     use_logits_soft_cap: bool = False,
     use_fp16_qk_reduction: bool = False,
-):
+    fp8_enabled: bool = False,
+) -> JitSpec:
     kwargs = {
         "variant_decl": variant_decl,
         "variant_name": variant_name,
@@ -1139,7 +1199,7 @@ def gen_customize_batch_prefill_module(
     if backend == "auto":
         raise ValueError("backend should not be auto when jit_args is provided")
     elif backend == "fa2":
-        gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+        gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
         (additional_params_decl, additional_func_params, additional_params_setter) = (
             generate_additional_params(
                 additional_tensor_names,
@@ -1149,13 +1209,19 @@ def gen_customize_batch_prefill_module(
             )
         )
 
-        with open(FLASHINFER_CSRC_DIR / "batch_prefill_customize_config.jinja") as f:
+        with open(
+            jit_env.FLASHINFER_CSRC_DIR / "batch_prefill_customize_config.jinja"
+        ) as f:
             config_templ = jinja2.Template(f.read())
 
-        with open(FLASHINFER_CSRC_DIR / "batch_prefill_paged_kernel_inst.jinja") as f:
+        with open(
+            jit_env.FLASHINFER_CSRC_DIR / "batch_prefill_paged_kernel_inst.jinja"
+        ) as f:
             paged_kernel_inst_templ = jinja2.Template(f.read())
 
-        with open(FLASHINFER_CSRC_DIR / "batch_prefill_ragged_kernel_inst.jinja") as f:
+        with open(
+            jit_env.FLASHINFER_CSRC_DIR / "batch_prefill_ragged_kernel_inst.jinja"
+        ) as f:
             ragged_kernel_inst_templ = jinja2.Template(f.read())
 
         kwargs |= {
@@ -1170,7 +1236,7 @@ def gen_customize_batch_prefill_module(
         os.makedirs(gen_directory, exist_ok=True)
 
         source_paths = []
-        for mask_mode in [0, 1, 2]:
+        for mask_mode in [0, 1, 2, 3]:
             dest_path = (
                 gen_directory / f"batch_prefill_paged_kernel_mask_{mask_mode}.cu"
             )
@@ -1195,7 +1261,7 @@ def gen_customize_batch_prefill_module(
             "batch_prefill.cu",
             "batch_prefill_jit_pybind.cu",
         ]:
-            src_path = FLASHINFER_CSRC_DIR / filename
+            src_path = jit_env.FLASHINFER_CSRC_DIR / filename
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
             with open(src_path, "r") as f:
@@ -1204,12 +1270,9 @@ def gen_customize_batch_prefill_module(
 
         generated_config_path = gen_directory / "batch_prefill_config.inc"
         write_if_different(generated_config_path, generated_inc_str)
-        return load_cuda_ops(
-            uri,
-            source_paths,
-        )
+        return gen_jit_spec(uri, source_paths)
     elif backend == "fa3":
-        gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+        gen_directory = jit_env.FLASHINFER_GEN_SRC_DIR / uri
         (additional_params_decl, additional_func_params, additional_params_setter) = (
             generate_additional_params(
                 additional_tensor_names,
@@ -1220,19 +1283,23 @@ def gen_customize_batch_prefill_module(
             )
         )
 
-        with open(
-            FLASHINFER_CSRC_DIR / "batch_prefill_sm90_customize_config.jinja"
-        ) as f:
+        _file_config = "batch_prefill_sm90_customize_config.jinja"
+        if fp8_enabled:
+            _file_paged_kernel_inst = "batch_prefill_fp8_paged_sm90_kernel_inst.jinja"
+            _file_ragged_kernel_inst = "batch_prefill_fp8_ragged_sm90_kernel_inst.jinja"
+            _file_csrc = "batch_prefill_fp8_sm90.cu"
+        else:
+            _file_paged_kernel_inst = "batch_prefill_paged_sm90_kernel_inst.jinja"
+            _file_ragged_kernel_inst = "batch_prefill_ragged_sm90_kernel_inst.jinja"
+            _file_csrc = "batch_prefill_sm90.cu"
+
+        with open(jit_env.FLASHINFER_CSRC_DIR / _file_config) as f:
             config_templ = jinja2.Template(f.read())
 
-        with open(
-            FLASHINFER_CSRC_DIR / "batch_prefill_paged_sm90_kernel_inst.jinja"
-        ) as f:
+        with open(jit_env.FLASHINFER_CSRC_DIR / _file_paged_kernel_inst) as f:
             paged_kernel_inst_templ = jinja2.Template(f.read())
 
-        with open(
-            FLASHINFER_CSRC_DIR / "batch_prefill_ragged_sm90_kernel_inst.jinja"
-        ) as f:
+        with open(jit_env.FLASHINFER_CSRC_DIR / _file_ragged_kernel_inst) as f:
             ragged_kernel_inst_templ = jinja2.Template(f.read())
 
         kwargs |= {
@@ -1243,7 +1310,7 @@ def gen_customize_batch_prefill_module(
         generated_inc_str = config_templ.render(**kwargs)
 
         source_paths = []
-        for mask_mode in [0, 1, 2]:
+        for mask_mode in [0, 1, 2, 3]:
             filename = f"batch_prefill_paged_sm90_kernel_mask_{mask_mode}.cu"
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
@@ -1263,10 +1330,10 @@ def gen_customize_batch_prefill_module(
             write_if_different(dest_path, source)
 
         for filename in [
-            "batch_prefill_sm90.cu",
+            _file_csrc,
             "batch_prefill_sm90_jit_pybind.cu",
         ]:
-            src_path = FLASHINFER_CSRC_DIR / filename
+            src_path = jit_env.FLASHINFER_CSRC_DIR / filename
             dest_path = gen_directory / filename
             source_paths.append(dest_path)
             with open(src_path, "r") as f:
@@ -1275,10 +1342,82 @@ def gen_customize_batch_prefill_module(
 
         generated_config_path = gen_directory / "batch_prefill_sm90_config.inc"
         write_if_different(generated_config_path, generated_inc_str)
-        return load_cuda_ops(
+        return gen_jit_spec(
             uri,
             source_paths,
-            extra_cuda_cflags=["-gencode=arch=compute_90a,code=sm_90a"],
+            extra_cuda_cflags=sm90a_nvcc_flags,
         )
     else:
         raise ValueError(f"Invalid backend: {backend}")
+
+
+def get_fmha_cutlass_sm100a_uri(
+    dtype_q: torch.dtype,
+    dtype_kv: torch.dtype,
+    dtype_o: torch.dtype,
+    dtype_idx: torch.dtype,
+    head_dim_qk: int,
+    head_dim_vo: int,
+    pos_encoding_mode: int,
+    use_sliding_window: bool,
+    use_logits_soft_cap: bool,
+) -> str:
+    # NOTE(Zihao): use different uri after when support customize attention
+    return "fmha_cutlass_sm100a"
+    # return (
+    #     f"fmha_cutlass_sm100a_dtype_q_{filename_safe_dtype_map[dtype_q]}_"
+    #     f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
+    #     f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
+    #     f"dtype_idx_{filename_safe_dtype_map[dtype_idx]}_"
+    #     f"head_dim_qk_{head_dim_qk}_"
+    #     f"head_dim_vo_{head_dim_vo}_"
+    #     f"posenc_{pos_encoding_mode}_"
+    #     f"use_swa_{use_sliding_window}_"
+    #     f"use_logits_cap_{use_logits_soft_cap}"
+    # )
+
+
+def gen_fmha_cutlass_sm100a_module(
+    dtype_q: torch.dtype,
+    dtype_kv: torch.dtype,
+    dtype_o: torch.dtype,
+    dtype_idx: torch.dtype,
+    head_dim_qk: int,
+    head_dim_vo: int,
+    pos_encoding_mode: int,
+    use_sliding_window: bool,
+    use_logits_soft_cap: bool,
+) -> JitSpec:
+    uri = get_fmha_cutlass_sm100a_uri(
+        dtype_q,
+        dtype_kv,
+        dtype_o,
+        dtype_idx,
+        head_dim_qk,
+        head_dim_vo,
+        pos_encoding_mode,
+        use_sliding_window,
+        use_logits_soft_cap,
+    )
+
+    source_paths = [
+        jit_env.FLASHINFER_CSRC_DIR / "fmha_cutlass_sm100.cu",
+        jit_env.FLASHINFER_CSRC_DIR / "fmha_cutlass_sm100_pybind.cu",
+        jit_env.FLASHINFER_CSRC_DIR / "blackwell_fmha_plan.cu",
+    ]
+    return gen_jit_spec(
+        uri,
+        source_paths,
+        extra_cuda_cflags=sm100a_nvcc_flags,
+    )
+
+
+def trtllm_fmha_gen_module():
+    return gen_jit_spec(
+        "fmha_gen",
+        [
+            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fmha_runner.cu",
+            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fmha_kernel_launcher.cu",
+        ],
+        extra_ldflags=["-lcuda"],
+    )
