@@ -14,6 +14,10 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+# NOTE(lequn): Do not "from .jit.env import xxx".
+# Do "from .jit import env as jit_env" and use "jit_env.xxx" instead.
+# This helps AOT script to override envs.
+
 import os
 import pathlib
 import re
@@ -21,6 +25,12 @@ import warnings
 
 from torch.utils.cpp_extension import _get_cuda_arch_flags
 
+FLASHINFER_BASE_DIR = pathlib.Path(
+    os.getenv("FLASHINFER_WORKSPACE_BASE", pathlib.Path.home().as_posix())
+)
+
+FLASHINFER_CACHE_DIR = FLASHINFER_BASE_DIR / ".cache" / "flashinfer"
+
 
 def _get_workspace_dir_name() -> pathlib.Path:
     try:
@@ -33,11 +43,8 @@ def _get_workspace_dir_name() -> pathlib.Path:
         arch = "_".join(sorted(set(re.findall(r"compute_(\d+)", "".join(flags)))))
     except Exception:
         arch = "noarch"
-    flashinfer_base = os.getenv(
-        "FLASHINFER_WORKSPACE_BASE", pathlib.Path.home().as_posix()
-    )
     # e.g.: $HOME/.cache/flashinfer/75_80_89_90/
-    return pathlib.Path(flashinfer_base) / ".cache" / "flashinfer" / arch
+    return FLASHINFER_CACHE_DIR / arch
 
 
 # use pathlib
@@ -45,10 +52,14 @@ FLASHINFER_WORKSPACE_DIR = _get_workspace_dir_name()
 FLASHINFER_JIT_DIR = FLASHINFER_WORKSPACE_DIR / "cached_ops"
 FLASHINFER_GEN_SRC_DIR = FLASHINFER_WORKSPACE_DIR / "generated"
 _package_root = pathlib.Path(__file__).resolve().parents[1]
+FLASHINFER_DATA = _package_root / "data"
 FLASHINFER_INCLUDE_DIR = _package_root / "data" / "include"
 FLASHINFER_CSRC_DIR = _package_root / "data" / "csrc"
+# FLASHINFER_SRC_DIR = _package_root / "data" / "src"
 FLASHINFER_TVM_BINDING_DIR = _package_root / "data" / "tvm_binding"
+FLASHINFER_AOT_DIR = _package_root / "data" / "aot"
 CUTLASS_INCLUDE_DIRS = [
     _package_root / "data" / "cutlass" / "include",
     _package_root / "data" / "cutlass" / "tools" / "util" / "include",
 ]
+SPDLOG_INCLUDE_DIR = _package_root / "data" / "spdlog" / "include"
