@@ -14,20 +14,65 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-import functools
-from types import SimpleNamespace
-from typing import List, Literal, Optional, Tuple, Union, overload
+from typing import Literal, Optional, Tuple, Union, overload
 
 import torch
 
-from .jit import gen_batch_mla_module
-from .utils import (
-    MaskMode,
-    _check_shape_dtype_device,
-    determine_mla_backend,
-    register_custom_op,
-    register_fake_op,
-)
+from .jit import JitSpec
+from .jit import env as jit_env
+from .jit import gen_batch_mla_module, gen_jit_spec, sm100a_nvcc_flags
+from .utils import MaskMode, _check_shape_dtype_device, determine_mla_backend
+
+
+def _check_cutlass_shape(q_nope_pe, ckv_kpe_cache, kv_len, page_table):
+    if q_nope_pe.ndim != 3:
+        raise ValueError(f"Expected q_nope_pe.ndim == 3, got {q_nope_pe.ndim}")
+    if ckv_kpe_cache.ndim != 3:
+        raise ValueError(f"Expected ckv_kpe_cache.ndim == 3, got {ckv_kpe_cache.ndim}")
+    if kv_len.ndim != 1:
+        raise ValueError(f"Expected kv_len.ndim == 1, got {kv_len.ndim}")
+    if page_table.ndim != 2:
+        raise ValueError(f"Expected page_table.ndim == 2, got {page_table.ndim}")
+    B_q, H, D_q = q_nope_pe.shape
+    D_ckv = ckv_kpe_cache.shape[2]
+    if H != 128:
+        raise ValueError(f"Expected 128 heads for q_nope_pe, got {H}")
+    if D_q != D_ckv or D_q != 576:
+        raise ValueError(
+            f"Expected head dim 576 for q_nope_pe and ckv_kpe_cache, got {D_q} and {D_ckv}"
+        )
+    B_block_table, block_num = page_table.shape
+    block_size = ckv_kpe_cache.shape[1]
+    if B_q != B_block_table:
+        raise ValueError(
+            f"Expected batch size {B_q} for q_nope_pe and block_table, got {B_q} and {B_block_table}"
+        )
+    if block_num % (128 / block_size) != 0:
+        raise ValueError(
+            f"Expected block_num % (128 / block_size) == 0, got {block_num=} and {block_size=}"
+        )
+
+
+_mla_module = None
+
+
+def gen_mla_module() -> JitSpec:
+    return gen_jit_spec(
+        "mla",
+        [
+            jit_env.FLASHINFER_CSRC_DIR / "cutlass_mla.cu",
+            jit_env.FLASHINFER_CSRC_DIR / "flashinfer_mla_ops.cu",
+        ],
+        extra_cuda_cflags=sm100a_nvcc_flags,
+    )
+
+
+def get_mla_module():
+    global _mla_module
+    if _mla_module is None:
+        _mla_module = gen_mla_module().build_and_load()
+    return _mla_module
+
 
 _batch_mla_modules = {}
 _batch_mla_sm90_modules = {}
@@ -40,7 +85,7 @@ def get_batch_mla_module(backend):
             _batch_mla_modules if backend == "fa2" else _batch_mla_sm90_modules
         )
         if args not in modules_dict:
-            modules_dict[args] = gen_batch_mla_module(backend, *args)
+            modules_dict[args] = gen_batch_mla_module(backend, *args).build_and_load()
         return modules_dict[args]
 
     return backend_module
@@ -152,10 +197,17 @@ class BatchMLAPagedAttentionWrapper:
         backend : str
             The implementation backend, could be ``auto``/``fa2`` or ``fa3``. Defaults to ``auto``.
             If set to ``auto``, the function will automatically choose the backend based on the
-            device architecture and kernel availability.
+            device architecture and kernel availability. If ``cutlass`` is provided, the MLA
+            kernels will be generated by CUTLASS and only float_workspace_buffer is required and
+            other arguments are ignored.
         """
         self._float_workspace_buffer = float_workspace_buffer
         self.device = float_workspace_buffer.device
+
+        if backend == "cutlass":
+            self._backend = backend
+            return
+
         self._int_workspace_buffer = torch.empty(
             (8 * 1024 * 1024,), dtype=torch.uint8, device=self.device
         )
@@ -294,6 +346,8 @@ class BatchMLAPagedAttentionWrapper:
         lse: Optional[torch.Tensor] = None,
         return_lse: bool = False,
         profiler_buffer: Optional[torch.Tensor] = None,
+        kv_len: Optional[torch.Tensor] = None,
+        page_table: Optional[torch.Tensor] = None,
     ) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
         r"""Run the MLA attention computation.
 
@@ -317,7 +371,40 @@ class BatchMLAPagedAttentionWrapper:
             Whether to return the log-sum-exp value, default is False.
         profiler_buffer : Optional[torch.Tensor]
             The buffer to store the profiler data.
+        kv_len : Optional[torch.Tensor]
+            The query length of each request, shape: ``[batch_size]``. Required when ``backend`` is ``cutlass``.
+        page_table : Optional[torch.Tensor]
+            The page table of the paged kv-cache, shape: ``[batch_size, num_pages]``. Required when ``backend`` is ``cutlass``.
         """
+        if self._backend == "cutlass":
+            if return_lse:
+                raise ValueError("return_lse does not support cutlass backend for now.")
+            if profiler_buffer is not None:
+                raise ValueError(
+                    "profiler_buffer does not support cutlass backend for now."
+                )
+            self._cached_module = get_mla_module()
+            if out is None:
+                out = torch.empty_like(q_nope)
+            else:
+                _check_shape_dtype_device(
+                    out, q_nope.shape, q_nope.dtype, q_nope.device, "out"
+                )
+            q_nope_pe = torch.cat([q_nope, q_pe], dim=-1)
+            ckv_kpe_cache = torch.cat([ckv_cache, kpe_cache], dim=-1)
+            _check_cutlass_shape(q_nope_pe, ckv_kpe_cache, kv_len, page_table)
+            lse = torch.empty(0, dtype=torch.float32, device=self.device)
+            self._cached_module.cutlass_mla_paged_attention.default(
+                self._float_workspace_buffer,
+                out,
+                lse,
+                q_nope_pe,
+                ckv_kpe_cache,
+                kv_len,
+                page_table,
+            )
+            return out
+
         if profiler_buffer is None:
             if self._use_profiler:
                 raise ValueError(
