@@ -19,27 +19,28 @@ from typing import Any, Tuple
 
 import torch
 
-from .jit import FLASHINFER_CSRC_DIR, has_prebuilt_ops, load_cuda_ops
+from .jit import JitSpec
+from .jit import env as jit_env
+from .jit import gen_jit_spec
 from .utils import register_custom_op, register_fake_op
 
 _quantization_module = None
 
 
+def gen_quantization_module() -> JitSpec:
+    return gen_jit_spec(
+        "quantization",
+        [
+            jit_env.FLASHINFER_CSRC_DIR / "quantization.cu",
+            jit_env.FLASHINFER_CSRC_DIR / "flashinfer_quantization_ops.cu",
+        ],
+    )
+
+
 def get_quantization_module():
     global _quantization_module
     if _quantization_module is None:
-        if has_prebuilt_ops:
-            _kernels = torch.ops.flashinfer_kernels
-
-            _quantization_module = _kernels
-        else:
-            _quantization_module = load_cuda_ops(
-                "quantization",
-                [
-                    FLASHINFER_CSRC_DIR / "quantization.cu",
-                    FLASHINFER_CSRC_DIR / "flashinfer_quantization_ops.cu",
-                ],
-            )
+        _quantization_module = gen_quantization_module().build_and_load()
     return _quantization_module
 
 
