@@ -19,27 +19,28 @@ from typing import Any, Optional, Tuple
 
 import torch
 
-from .jit import FLASHINFER_CSRC_DIR, has_prebuilt_ops, load_cuda_ops
+from .jit import JitSpec
+from .jit import env as jit_env
+from .jit import gen_jit_spec
 from .utils import register_custom_op, register_fake_op
 
 _rope_module = None
 
 
+def gen_rope_module() -> JitSpec:
+    return gen_jit_spec(
+        "rope",
+        [
+            jit_env.FLASHINFER_CSRC_DIR / "rope.cu",
+            jit_env.FLASHINFER_CSRC_DIR / "flashinfer_rope_ops.cu",
+        ],
+    )
+
+
 def get_rope_module():
     global _rope_module
     if _rope_module is None:
-        if has_prebuilt_ops:
-            _kernels = torch.ops.flashinfer_kernels
-
-            _rope_module = _kernels
-        else:
-            _rope_module = load_cuda_ops(
-                "rope",
-                [
-                    FLASHINFER_CSRC_DIR / "rope.cu",
-                    FLASHINFER_CSRC_DIR / "flashinfer_rope_ops.cu",
-                ],
-            )
+        _rope_module = gen_rope_module().build_and_load()
     return _rope_module
 
 
@@ -298,7 +299,7 @@ def apply_rope_inplace(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
@@ -384,7 +385,7 @@ def apply_rope_pos_ids_inplace(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
@@ -446,7 +447,7 @@ def apply_llama31_rope_inplace(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
@@ -553,7 +554,7 @@ def apply_llama31_rope_pos_ids_inplace(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
@@ -629,7 +630,7 @@ def apply_rope(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
@@ -738,7 +739,7 @@ def apply_rope_pos_ids(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
@@ -810,7 +811,7 @@ def apply_llama31_rope(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
@@ -931,7 +932,7 @@ def apply_llama31_rope_pos_ids(
     segment the query of the i-th segment is ``q[indptr[i]:indptr[i+1]]`` and the key of the
     i-th segment is ``k[indptr[i]:indptr[i+1]]``, the first element of :attr:`indptr` is always
     0 and the last element of :attr:`indptr` is the total number of queries/keys in the batch.
-    Please see :ref:`Ragged Tensor tutorial <ragged-layout>` for more details about the
+    Please see :ref:`Ragged Tensor tutorial <kv-layout>` for more details about the
     ragged tensor.
 
     Parameters
