@@ -19,28 +19,62 @@ from typing import Optional, Union
 
 import torch
 
-from .jit import FLASHINFER_CSRC_DIR, has_prebuilt_ops, load_cuda_ops
+from .jit import JitSpec
+from .jit import env as jit_env
+from .jit import gen_jit_spec
 from .utils import register_custom_op, register_fake_op
 
 _sampling_module = None
 
 
+def gen_sampling_module() -> JitSpec:
+    return gen_jit_spec(
+        "sampling",
+        [
+            jit_env.FLASHINFER_CSRC_DIR / "sampling.cu",
+            jit_env.FLASHINFER_CSRC_DIR / "renorm.cu",
+            jit_env.FLASHINFER_CSRC_DIR / "flashinfer_sampling_ops.cu",
+        ],
+    )
+
+
 def get_sampling_module():
     global _sampling_module
     if _sampling_module is None:
-        if has_prebuilt_ops:
-            _kernels = torch.ops.flashinfer_kernels
+        module = gen_sampling_module().build_and_load()
 
-            module = _kernels
-        else:
-            module = load_cuda_ops(
-                "sampling",
-                [
-                    FLASHINFER_CSRC_DIR / "sampling.cu",
-                    FLASHINFER_CSRC_DIR / "renorm.cu",
-                    FLASHINFER_CSRC_DIR / "flashinfer_sampling_ops.cu",
-                ],
+        # torch library for sampling_from_logits
+        @register_custom_op("flashinfer::sampling_from_logits", mutates_args=())
+        def sampling_from_logits(
+            logits: torch.Tensor,
+            indices: Optional[torch.Tensor],
+            deterministic: bool,
+            generator: Optional[torch.Generator],
+        ) -> torch.Tensor:
+            device = logits.device
+            # TODO: support more data types in logits to avoid conversion
+            # to float32
+            logits = logits.float()
+            batch_size = indices.size(0) if indices is not None else logits.size(0)
+            samples = torch.empty(batch_size, dtype=torch.int32, device=device)
+            module.sampling_from_logits.default(
+                logits,
+                samples,
+                indices,
+                deterministic,
+                generator,
             )
+            return samples
+
+        @register_fake_op("flashinfer::sampling_from_logits")
+        def _fake_sampling_from_logits(
+            logits: torch.Tensor,
+            indices: Optional[torch.Tensor],
+            deterministic: bool,
+            generator: Optional[torch.Generator],
+        ) -> torch.Tensor:
+            batch_size = indices.size(0) if indices is not None else logits.size(0)
+            return torch.empty(batch_size, dtype=torch.int32, device=logits.device)
 
         # torch library for sampling_from_probs
 
@@ -64,6 +98,8 @@ def get_sampling_module():
             )
             return samples
 
+        # torch library for sampling_from_probs
+
         @register_fake_op("flashinfer::sampling_from_probs")
         def _fake_sampling_from_probs(
             probs: torch.Tensor,
@@ -384,6 +420,7 @@ def get_sampling_module():
         # Register the module
         _sampling_module = SimpleNamespace(
             sampling_from_probs=sampling_from_probs,
+            sampling_from_logits=sampling_from_logits,
             top_p_sampling_from_probs=top_p_sampling_from_probs,
             top_k_sampling_from_probs=top_k_sampling_from_probs,
             min_p_sampling_from_probs=min_p_sampling_from_probs,
@@ -404,6 +441,64 @@ def _to_tensor_scalar_tuple(x):
         return (None, x)
 
 
+def sampling_from_logits(
+    logits: torch.Tensor,
+    indices: Optional[torch.Tensor] = None,
+    deterministic: bool = True,
+    generator: Optional[torch.Generator] = None,
+    check_nan: bool = False,
+) -> torch.Tensor:
+    r"""Fused GPU kernel for category sampling from logits. It's equivalent to sampling
+    from :attr:`logits` after applying softmax.
+    Parameters
+    ----------
+    logits: torch.Tensor
+        Logits for sampling. When indices is not provided, shape should be ``(batch_size, num_classes)``
+        and the i-th output will be sampled from the i-th row of logits. When indices is provided,
+        shape should be ``(unique_batch_size, num_classes)`` where unique_batch_size is the number of unique
+        probability distributions.
+    indices: Optional[torch.Tensor]
+        Optional indices tensor of shape ``(batch_size,)`` that maps each output to a row in logits.
+        For example, if indices[i] = j, then the i-th output will be sampled from logits[j].
+        This allows reusing the same probability distribution for multiple outputs.
+        If indices is not provided, the i-th output will be sampled from the i-th row of logits.
+    deterministic: bool
+        Since the sampling doesn't use cub's BlockScan, the sampling is deterministic. We keep this
+        argument for compatibility with other sampling functions.
+    generator: Optional[torch.Generator]
+        A random number generator for the operation.
+    check_nan: bool
+        Whether to check nan in :attr:`logits`, default is ``False``.
+    Returns
+    -------
+    samples: torch.Tensor
+        Sampled categories, shape (batch_size,). It's equivalent to sampling from
+        :attr:`logits` after applying softmax.
+    Examples
+    --------
+    >>> import torch
+    >>> import flashinfer
+    >>> torch.manual_seed(42)
+    >>> batch_size = 4
+    >>> vocab_size = 5
+    >>> logits = torch.rand(batch_size, vocab_size).to(0)
+    >>> logits
+    tensor([[0.8823, 0.9150, 0.3829, 0.9593, 0.3904],
+            [0.6009, 0.2566, 0.7936, 0.9408, 0.1332],
+            [0.9346, 0.5936, 0.8694, 0.5677, 0.7411],
+            [0.4294, 0.8854, 0.5739, 0.2666, 0.6274]], device='cuda:0')
+    >>> samples = flashinfer.sampling.sampling_from_logits(logits)
+    >>> samples
+    tensor([0, 1, 1, 1], device='cuda:0', dtype=torch.int32)
+    """
+    if check_nan:
+        if torch.any(torch.isnan(logits)):
+            raise ValueError("Input logits contains NaN.")
+    return get_sampling_module().sampling_from_logits(
+        logits, indices, deterministic, generator
+    )
+
+
 def sampling_from_probs(
     probs: torch.Tensor,
     indices: Optional[torch.Tensor] = None,
