@@ -14,6 +14,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import functools
 import math
 import os
 from enum import Enum
@@ -24,6 +25,9 @@ import torch.version
 from torch.torch_version import TorchVersion
 from torch.torch_version import __version__ as torch_version
 
+from .jit import env as jit_env
+from .jit import gen_jit_spec
+
 IS_BUILDING_DOCS = os.environ.get("FLASHINFER_BUILDING_DOCS") == "1"
 
 
@@ -37,6 +41,7 @@ class MaskMode(Enum):
     NON_CAUSAL = 0
     CAUSAL = 1
     CUSTOM = 2
+    MULTIITEMSCORING = 3
 
 
 class TensorLayout(Enum):
@@ -300,8 +305,43 @@ def is_fa3_backend_supported(
         return False
     if use_fp16_qk_reductions:
         return False
-    # NOTE: currently fp8 is not supported in our FA3 backend
-    # will add support soon
+    return True
+
+
+def is_cutlass_backend_supported(
+    pos_encoding_mode: int,
+    use_fp16_qk_reductions: bool,
+    use_custom_mask: bool,
+    dtype_q: torch.dtype,
+    dtype_kv: torch.dtype,
+) -> bool:
+    """
+    Check if the cutlass backend is supported based on the given parameters.
+
+    Parameters
+    ----------
+    pos_encoding_mode : int
+        The positional encoding mode.
+    use_fp16_qk_reductions : bool
+        Whether FP16 QK reductions are allowed.
+    use_custom_mask : bool
+        Whether a custom mask is used.
+    dtype_q : torch.dtype
+        The data type of the query tensor.
+    dtype_kv : torch.dtype
+        The data type of the key-value tensor.
+
+    Returns
+    -------
+    bool
+        True if the cutlass backend is supported, False otherwise.
+    """
+    if use_custom_mask:
+        return False
+    if pos_encoding_mode != PosEncodingMode.NONE.value:
+        return False
+    if use_fp16_qk_reductions:
+        return False
     if dtype_q in [torch.float8_e4m3fn, torch.float8_e5m2]:
         return False
     if dtype_kv in [torch.float8_e4m3fn, torch.float8_e5m2]:
@@ -354,9 +394,20 @@ def determine_attention_backend(
         return "fa2"
 
 
+def version_at_least(version: str, base_version: str) -> bool:
+    from packaging import version as pkg_version
+
+    return pkg_version.parse(version) >= pkg_version.parse(base_version)
+
+
 def is_sm90a_supported(device: torch.device) -> bool:
     major, _ = get_compute_capability(device)
-    return major == 9 and torch.version.cuda >= "12.3"
+    return major == 9 and version_at_least(torch.version.cuda, "12.3")
+
+
+def is_sm100a_supported(device: torch.device) -> bool:
+    major, _ = get_compute_capability(device)
+    return major == 10 and version_at_least(torch.version.cuda, "12.8")
 
 
 def determine_mla_backend(device: torch.device) -> str:
@@ -382,3 +433,74 @@ def _check_shape_dtype_device(
         raise ValueError(
             f"Invalid device of {name}: expected {expected_device}, got {x.device}"
         )
+
+
+def get_logging_module():
+    return gen_jit_spec(
+        "logging",
+        [
+            jit_env.FLASHINFER_CSRC_DIR / "logging.cc",
+        ],
+        extra_include_paths=[
+            jit_env.SPDLOG_INCLUDE_DIR,
+            jit_env.FLASHINFER_INCLUDE_DIR,
+        ],
+    ).build_and_load()
+
+
+class LogLevel(Enum):
+    TRACE = 0
+    DEBUG = 1
+    INFO = 2
+    WARN = 3
+    ERROR = 4
+    CRITICAL = 5
+
+
+log_level_map = {
+    "trace": LogLevel.TRACE,
+    "debug": LogLevel.DEBUG,
+    "info": LogLevel.INFO,
+    "warn": LogLevel.WARN,
+    "error": LogLevel.ERROR,
+    "critical": LogLevel.CRITICAL,
+}
+
+
+def set_log_level(lvl_str: str) -> None:
+    get_logging_module().set_log_level(log_level_map[lvl_str].value)
+
+
+@functools.cache
+def get_trtllm_utils_module():
+    return gen_jit_spec(
+        "trtllm_utils",
+        [
+            jit_env.FLASHINFER_CSRC_DIR
+            / "nv_internal/tensorrt_llm/kernels/delayStream.cu",
+        ],
+        extra_include_paths=[
+            jit_env.FLASHINFER_CSRC_DIR / "nv_internal",
+            jit_env.FLASHINFER_CSRC_DIR / "nv_internal" / "include",
+            jit_env.FLASHINFER_CSRC_DIR
+            / "nv_internal"
+            / "tensorrt_llm"
+            / "cutlass_extensions"
+            / "include",
+            jit_env.FLASHINFER_CSRC_DIR
+            / "nv_internal"
+            / "tensorrt_llm"
+            / "kernels"
+            / "internal_cutlass_kernels"
+            / "include",
+            jit_env.FLASHINFER_CSRC_DIR
+            / "nv_internal"
+            / "tensorrt_llm"
+            / "kernels"
+            / "internal_cutlass_kernels",
+        ],
+    ).build_and_load()
+
+
+def delay_kernel(stream_delay_micro_secs):
+    get_trtllm_utils_module().delay_kernel(stream_delay_micro_secs)
